from dataclasses import dataclass, field

from emodpy.reporters.base import CustomReporter


@dataclass
class Report_TBHIV_ByAge(CustomReporter):
    name: str = field(default="Report_TBHIV_ByAge")
    dll_file: str = field(default="lib_customreport_TBHIV_ReportByAge.dll")

    def add_report(self, Max_Age_Yrs, Min_Age_Yrs, Start_Year, Stop_Year):
        self._add_report({
            "Max_Age_Yrs": Max_Age_Yrs,
            "Min_Age_Yrs": Min_Age_Yrs,
            "Start_Year": Start_Year,
            "Stop_Year": Stop_Year
        })


@dataclass
class ReportHumanMigrationTracking(CustomReporter):
    name: str = field(default="ReportHumanMigrationTracking")
    dll_file: str = field(default="libhumanmigrationtracking.dll")
    Reports: list = field(default_factory=lambda: [{}])


@dataclass
class ReportNodeDemographics(CustomReporter):
    name: str = field(default="ReportNodeDemographics")
    age_bins: list = field(default_factory=list)
    dll_file: str = field(default="libReportNodeDemographics.dll")

    def from_dict(self, data):
        self.age_bins = [r["Age_Bins"] for r in data["Reports"]]

    def add_report(self, age_bins):
        self.age_bins.append(age_bins)

    def to_dict(self):
        self.Reports = [{
            "Age_Bins": ab
        } for ab in self.age_bins]
        return super().to_dict()


@dataclass
class MalariaSummaryReport(CustomReporter):
    name: str = field(default="MalariaSummaryReport")
    dll_file: str = "libmalariasummary_report_plugin.dll"

    def add_report(self, Age_Bins=[], Duration_Days=365, Event_Trigger_List=[], Individual_Property_Filter="",
                   Infectiousness_Bins=[], Max_Number_Reports=10000, Parasitemia_Bins=[], Report_Description="",
                   Reporting_Interval=30, Start_Day=0):
        self._add_report({
            "Age_Bins": Age_Bins,
            "Duration_Days": Duration_Days,
            "Event_Trigger_List": Event_Trigger_List,
            "Individual_Property_Filter": Individual_Property_Filter,
            "Infectiousness_Bins": Infectiousness_Bins,
            "Max_Number_Reports": Max_Number_Reports,
            "Parasitemia_Bins": Parasitemia_Bins,
            "Report_Description": Report_Description,
            "Reporting_Interval": Reporting_Interval,
            "Start_Day": Start_Day
        })


@dataclass
class ReportEventCounter(CustomReporter):
    name: str = field(default="ReportEventCounter")
    dll_file: str = field(default="libreporteventcounter.dll")

    def add_report(self, Duration_Days=10000, Event_Trigger_List=[], Nodeset_Config={"class": "NodeSetAll"},
                   Report_Description="", Start_Day=0):
        self._add_report({
            "Duration_Days": Duration_Days,
            "Event_Trigger_List": Event_Trigger_List,
            "Nodeset_Config": Nodeset_Config,
            "Report_Description": Report_Description,
            "Start_Day": Start_Day
        })


@dataclass
class ReportMalariaFiltered(CustomReporter):
    name: str = field(default="ReportMalariaFiltered")
    dll_file: str = field(default="libReportMalariaFiltered.dll")

    def add_report(self, End_Day=0, Node_IDs_Of_Interest=[], Report_File_Name="ReportMalariaFiltered.json",
                   Start_Day=0):
        self._add_report({
            "End_Day": End_Day,
            "Node_IDs_Of_Interest": Node_IDs_Of_Interest,
            "Report_File_Name": Report_File_Name,
            "Start_Day": Start_Day
        })
